/* parameters.cpp
 * Parameters class. Input file reader.
 */
#include "parameters.hpp"
#include "stringTools.hpp"
#include <iostream>
#include <sstream>
#include <fstream>

using std::string;
using std::vector;

// ======================================================================
// Class Parameters
// ======================================================================
// PUBLIC MEMBERS
// ======================================================================

// ======================================================================
// Construct, assign

Parameters::Parameters() {}

Parameters::Parameters(string fileName) {
  _fileName = fileName;
  readInputFile();
}

Parameters::Parameters(const Parameters& that) {
  this->_fileName   = that._fileName;
  // We do not use readInputFile as the config 
  // may have been changed manually...
  this->_parameters = that._parameters;
}

Parameters& Parameters::operator=(Parameters& that) {
  if (this!=&that) {
    _fileName   = that._fileName;
    _parameters = that._parameters;
  }
  return *this;
}

// ======================================================================
// Get values from configuration

// ...

// =====================================================================
// Modify configuration

// ...

// =====================================================================
// Quick info

// ...

// =====================================================================
// I/O

// ...

// ======================================================================
// PRIVATE MEMBERS
// ======================================================================

void Parameters::readInputFile(bool clear) {

  if (clear) _parameters.clear();
 
  std::ifstream pfile(_fileName.c_str());
  if (!pfile.is_open())
    std::cerr << "Couldn't open parameters file: " << _fileName     << std::endl 
              << "           Will use default values if available." << std::endl;

  vector<string> lines;
  string         l,key,value,KEY;
  // Store all the lines of the file
  while(getline(pfile,l)) 
    lines.push_back(l);

  // Loop on lines
  for (string& line:lines) {
    // The line must have an = sign and must not be a comment
    if (line.find('=')!=line.npos and line.at(0)!='#') {
      std::stringstream ss(line);
      getline(ss,key,'=');
      getline(ss,value);

      key   = stringTrim(key);
      KEY   = toUpper(key);
      value = stringTrim(value);

      if (KEY.length()==0)
        std::cerr << "Warning: Empty key detected in configuration file" << std::endl;
      if (value.length()==0)
        std::cerr << "Warning: Empty value for key \""<<key<<"\"" << std::endl;
      if (_parameters.find(KEY) != _parameters.end())
        std::cerr << "Warning: Key \"" << key << "\" found several times in parameters file." << std::endl
                  << "         The value begin read last will be used." << std::endl;
    
      _parameters.insert(std::make_pair(KEY,value)); 

    }
  }

}

void Parameters::errNoKey(string key) const {
  std::cerr << "Error : no key \"" << key << "\" in configuration file and no default value provided." << std::endl;
  abort();
}

