/* parameters.hpp
 * Parameters class. Input file reader.
 */
#ifndef _PARAMETERS_HPP
#define _PARAMETERS_HPP

#include <map>
#include <vector>

/* Manages options for simulations.
 *  
 * Options are usually read from an input file, but can be set manually.
 * The input line format is the following:
 *
 * keyword = a list of options
 *
 * If the line ends with a backslash character (\),
 * options can be continued on the next line
 *
 */
class Parameters {

  public :

    Parameters();

    /* Constructor with a file name
     * fileName can be absolute or relative.
     */
    Parameters(std::string);

    /* Copy constructor
     * All options are copied from the configuration state, 
     * not the input file (in case some entries were added/modified)
     */
    Parameters(const Parameters& cfg);

    /* Assignment operator. 
     * All options are copied from the configuration state, 
     * not the input file (in case some entries were added/modified)
     */
    Parameters& operator=(Parameters& cfg);

    // Destructor. Nothing special to do.
   ~Parameters(){};

    // ===============================================================
    // Get values from config
    // ===============================================================
   
    // ...

    // ===============================================================
    // Modify configuration
    // ===============================================================

    // ...

    // ===============================================================
    // Quick information
    // ===============================================================

    // ...

    // ===============================================================
    // I/O
    // ===============================================================

    // ...

  private :
    
    /* Parses input file and store configuration.
     * clear: If true(default), previously defined options are deleted.
     *        When false, can be used to complete a configuration with another file.
     *        Previously existing entries are overwritten though.
     *  
     *  The input line format is the following:
     *
     *  keyword = a list of options
     *
     *  If the line ends with a backslash character (\\),
     *  options can be continued on the next line
     *
     */
    void readInputFile(bool clear=true);

    // Prints an error message if file misses a key
    void errNoKey(std::string key) const;

    std::string                       _fileName;   // input file
    std::map<std::string,std::string> _parameters; // configuration info
  
};

#endif /* _PARAMETERS_HPP */

